/*
 * fservcmd.c - Simple File Server Command Handling
 *
 * Copyright (c) 2000 Cogito, LLC 
 *
 * Cogito grants you ("Licensee") a non-exclusive, royalty free, license to
 * use, modify and redistribute this software in source and binary form,
 * provided that i) this copyright notice and license appear on all copies
 * of the software; and ii) Licensee does not utilize the software in a
 * manner which is disparaging to Cogito.
 *
 * This software is provided "AS IS," without warranty of any kind.
 *
 * 12/2000: Initial version.
 *          M. Tim Jones <mtj@cogitollc.com>
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>

#define BUF_SIZE 2048

/* Uncomment the following for HEX ASCII protocol (slower than binary) */
//#define HEXASCII_EMISSION


/* Handle a socket connection that has sent an $ls command.  The home path
 * for this user is also passed in given the remote connection has been
 * authenticated.  A simple popen is then used to gather the subdirectory
 * listing and the output is emitted through the socket connection.
 */
void handleRemoteList(int sockfd, char *line, char *subdir)
{
  FILE *fp;
  char buffer[BUF_SIZE+1];
  int  i, chars_read;

  memset(buffer, 0, sizeof(buffer));

  /* Ensure that the home path and user supplied subdirectory have a 
   * separating '/'.
   */
  if (line[0] == '/') {
    sprintf(buffer, "ls -l %s%s", subdir, &line[1]);
  } else {
    sprintf(buffer, "ls -l %s%s", subdir, line);
  }

  /* Strip any CR/LF from the stream (confuses popen...). */
  for (i = 0 ; i < strlen(buffer) ; i++) {
    if ((buffer[i] == 0x0a) || (buffer[i] == 0x0d)) buffer[i] = 0;
  }

  fp = popen(buffer, "r");
  if (fp != NULL) {
    memset(buffer, 0, sizeof(buffer));
    chars_read = fread(buffer, sizeof(char), BUF_SIZE, fp);
    while (chars_read > 0) {
      buffer[chars_read - 1] = 0;
      write(sockfd, buffer, chars_read);
      chars_read = fread(buffer, sizeof(char), BUF_SIZE, fp);
    }
    write(sockfd, "\n", 1);
    pclose(fp);
  }
}


/* Handle a socket connection that has sent an $get command.  The home path
 * for this user is also passed in given the remote connection has been
 * authenticated.  
 *
 * The following protocol is followed to pass the file through to the remote
 * client:
 *
 *    Get "$get <subdir><filename>"  <-- Determined from caller
 *    Emit <long size in hex ascii>
 *    Emit <hex ascii> for all bytes
 *    Emit <byte checksum in hex ascii >
 *
*/

void handleRemoteGet(int sockfd, char *line, char *subdir)
{
  FILE *fp;
  char buffer[BUF_SIZE+1];
  char temp[BUF_SIZE+1];
  int  i, chars_read;
  struct stat buf;
  signed char checksum = 0;
  int max_read;

  memset(buffer, 0, sizeof(buffer));

  /* Ensure that the home path and user supplied subdir/file have a 
   * separating '/'.
   */
  if (line[0] == '/') {
    sprintf(buffer, "%s%s", subdir, &line[1]);
  } else {
    sprintf(buffer, "%s%s", subdir, line);
  }

  /* Strip any CR/LF in the command stream */
  for (i = 0 ; i < strlen(buffer) ; i++) {
    if ((buffer[i] == 0x0a) || (buffer[i] == 0x0d)) buffer[i] = 0;
  }

  if (!lstat(buffer, &buf)) {

    /* Only support grabbing of regular files */
    if (S_ISREG(buf.st_mode)) {

      /* Emit the file size... */
      sprintf(temp, "%d\n", buf.st_size);
      write(sockfd, temp, strlen(temp));

      printf("Emitting file %s (size %d)\n", buffer, buf.st_size);

      fp = fopen(buffer, "rb");

#if defined(HEXASCII_EMISSION)

      /* Walk through all bytes in the file and emit as hex ascii */
      for (i = 0 ; i < buf.st_size ; i++) {
        fread(buffer, 1, 1, fp);
        checksum += buffer[0];
        sprintf(temp, "%d\n", (unsigned char)buffer[0]);
        write(sockfd, temp, strlen(temp));
      }

#else /* BINARY EMISSION */

      while (buf.st_size > 0) {
        int toread = (buf.st_size > BUF_SIZE) ? BUF_SIZE : buf.st_size;
        fread(buffer, 1, toread, fp);
        write(sockfd, buffer, toread);
        buf.st_size -= toread;

        // Update the checksum
        for (i = 0 ; i < toread ; i++) checksum += buffer[i];
      }

#endif

      fclose(fp);
      
      /* Emit the checksum */
      sprintf(temp, "%d\n", checksum);
      write(sockfd, temp, strlen(temp));

    } else {
      printf("Not a regular file\n");
    }

  } else {
    printf("No file found...\n");
  }

}


void handleRemotePut(int sockfd, char *line)
{
  // TODO
}

